/*
  * JBoss, Home of Professional Open Source
  * Copyright 2005, JBoss Inc., and individual contributors as indicated
  * by the @authors tag. See the copyright.txt in the distribution for a
  * full listing of individual contributors.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as
  * published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This software is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this software; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
  */
package org.jboss.jms.server;

import org.jboss.jms.server.connectionfactory.JNDIBindings;

/**
 * @author <a href="mailto:ovidiu@jboss.org">Ovidiu Feodorov</a>
 * @version <tt>$Revision: 1174 $</tt>
 *
 * $Id: ConnectionFactoryManager.java 1174 2006-08-02 14:14:32Z timfox $
 */
public interface ConnectionFactoryManager
{
   /**
    * @param jndiBindings - if null, the connection factory will be created and registered with the
    *        AOP subsystem, but not bound in JNDI.
    *
    * @return an identifier that uniques identifies the registered ConnectionFactory.
    */
   int registerConnectionFactory(String clientID, JNDIBindings jndiBindings,
                                 String locatorURI, boolean clientPing,
                                 int prefetchSize,
                                 int defaultTempQueueFullSize,
                                 int defaultTempQueuePageSize,
                                 int defaultTempQueueDownCacheSize) throws Exception;

   void unregisterConnectionFactory(int connectionFactoryID) throws Exception;

   javax.jms.ConnectionFactory getConnectionFactory(int connectionFactoryID);
}
